package java.util.prefs;

import java.util.StringTokenizer;
import java.io.ByteArrayOutputStream;
import sun.util.logging.PlatformLogger;

class WindowsPreferences extends AbstractPreferences
{
    private static PlatformLogger logger;
    private static final byte[] WINDOWS_ROOT_PATH;
    private static final int HKEY_CURRENT_USER = -2147483647;
    private static final int HKEY_LOCAL_MACHINE = -2147483646;
    private static final int USER_ROOT_NATIVE_HANDLE = -2147483647;
    private static final int SYSTEM_ROOT_NATIVE_HANDLE = -2147483646;
    private static final int MAX_WINDOWS_PATH_LENGTH = 256;
    private static volatile Preferences userRoot;
    private static volatile Preferences systemRoot;
    private static final int ERROR_SUCCESS = 0;
    private static final int ERROR_FILE_NOT_FOUND = 2;
    private static final int ERROR_ACCESS_DENIED = 5;
    private static final int NATIVE_HANDLE = 0;
    private static final int ERROR_CODE = 1;
    private static final int SUBKEYS_NUMBER = 0;
    private static final int VALUES_NUMBER = 2;
    private static final int MAX_KEY_LENGTH = 3;
    private static final int MAX_VALUE_NAME_LENGTH = 4;
    private static final int DISPOSITION = 2;
    private static final int REG_CREATED_NEW_KEY = 1;
    private static final int REG_OPENED_EXISTING_KEY = 2;
    private static final int NULL_NATIVE_HANDLE = 0;
    private static final int DELETE = 65536;
    private static final int KEY_QUERY_VALUE = 1;
    private static final int KEY_SET_VALUE = 2;
    private static final int KEY_CREATE_SUB_KEY = 4;
    private static final int KEY_ENUMERATE_SUB_KEYS = 8;
    private static final int KEY_READ = 131097;
    private static final int KEY_WRITE = 131078;
    private static final int KEY_ALL_ACCESS = 983103;
    private static int INIT_SLEEP_TIME;
    private static int MAX_ATTEMPTS;
    private boolean isBackingStoreAvailable;
    
    static Preferences getUserRoot() {
        Preferences preferences = WindowsPreferences.userRoot;
        if (preferences == null) {
            synchronized (WindowsPreferences.class) {
                preferences = WindowsPreferences.userRoot;
                if (preferences == null) {
                    preferences = (WindowsPreferences.userRoot = new WindowsPreferences(-2147483647, WindowsPreferences.WINDOWS_ROOT_PATH));
                }
            }
        }
        return preferences;
    }
    
    static Preferences getSystemRoot() {
        Preferences preferences = WindowsPreferences.systemRoot;
        if (preferences == null) {
            synchronized (WindowsPreferences.class) {
                preferences = WindowsPreferences.systemRoot;
                if (preferences == null) {
                    preferences = (WindowsPreferences.systemRoot = new WindowsPreferences(-2147483646, WindowsPreferences.WINDOWS_ROOT_PATH));
                }
            }
        }
        return preferences;
    }
    
    private static native int[] WindowsRegOpenKey(int p0, byte[] p1, int p2);
    
    private static int[] WindowsRegOpenKey1(int i, byte[] array, int n) {
        int[] array2 = WindowsRegOpenKey(i, array, n);
        if (array2[1] == 0) {
            return array2;
        }
        if (array2[1] == 2) {
            logger().warning("Trying to recreate Windows registry node " + byteArrayToString(array) + " at root 0x" + Integer.toHexString(i) + ".");
            WindowsRegCloseKey(WindowsRegCreateKeyEx(i, array)[0]);
            return WindowsRegOpenKey(i, array, n);
        }
        if (array2[1] != 5) {
            long millis = (long)WindowsPreferences.INIT_SLEEP_TIME;
            int j = 0;
            while (j < WindowsPreferences.MAX_ATTEMPTS) {
                try {
                    Thread.sleep(millis);
                }
                catch (InterruptedException ex) {
                    return array2;
                }
                millis *= 2L;
                array2 = WindowsRegOpenKey(i, array, n);
                if (array2[1] == 0) {
                    return array2;
                }
                ++j;
            }
            return array2;
        }
        return array2;
    }
    
    private static native int WindowsRegCloseKey(int p0);
    
    private static native int[] WindowsRegCreateKeyEx(int p0, byte[] p1);
    
    private static int[] WindowsRegCreateKeyEx1(int n, byte[] array) {
        int[] array2 = WindowsRegCreateKeyEx(n, array);
        if (array2[1] == 0) {
            return array2;
        }
        long millis = (long)WindowsPreferences.INIT_SLEEP_TIME;
        int i = 0;
        while (i < WindowsPreferences.MAX_ATTEMPTS) {
            try {
                Thread.sleep(millis);
            }
            catch (InterruptedException ex) {
                return array2;
            }
            millis *= 2L;
            array2 = WindowsRegCreateKeyEx(n, array);
            if (array2[1] == 0) {
                return array2;
            }
            ++i;
        }
        return array2;
    }
    
    private static native int WindowsRegDeleteKey(int p0, byte[] p1);
    
    private static native int WindowsRegFlushKey(int p0);
    
    private static int WindowsRegFlushKey1(int n) {
        int n2 = WindowsRegFlushKey(n);
        if (n2 == 0) {
            return n2;
        }
        long millis = (long)WindowsPreferences.INIT_SLEEP_TIME;
        int i = 0;
        while (i < WindowsPreferences.MAX_ATTEMPTS) {
            try {
                Thread.sleep(millis);
            }
            catch (InterruptedException ex) {
                return n2;
            }
            millis *= 2L;
            n2 = WindowsRegFlushKey(n);
            if (n2 == 0) {
                return n2;
            }
            ++i;
        }
        return n2;
    }
    
    private static native byte[] WindowsRegQueryValueEx(int p0, byte[] p1);
    
    private static native int WindowsRegSetValueEx(int p0, byte[] p1, byte[] p2);
    
    private static int WindowsRegSetValueEx1(int n, byte[] array, byte[] array2) {
        int n2 = WindowsRegSetValueEx(n, array, array2);
        if (n2 == 0) {
            return n2;
        }
        long millis = (long)WindowsPreferences.INIT_SLEEP_TIME;
        int i = 0;
        while (i < WindowsPreferences.MAX_ATTEMPTS) {
            try {
                Thread.sleep(millis);
            }
            catch (InterruptedException ex) {
                return n2;
            }
            millis *= 2L;
            n2 = WindowsRegSetValueEx(n, array, array2);
            if (n2 == 0) {
                return n2;
            }
            ++i;
        }
        return n2;
    }
    
    private static native int WindowsRegDeleteValue(int p0, byte[] p1);
    
    private static native int[] WindowsRegQueryInfoKey(int p0);
    
    private static int[] WindowsRegQueryInfoKey1(int n) {
        int[] array = WindowsRegQueryInfoKey(n);
        if (array[1] == 0) {
            return array;
        }
        long millis = (long)WindowsPreferences.INIT_SLEEP_TIME;
        int i = 0;
        while (i < WindowsPreferences.MAX_ATTEMPTS) {
            try {
                Thread.sleep(millis);
            }
            catch (InterruptedException ex) {
                return array;
            }
            millis *= 2L;
            array = WindowsRegQueryInfoKey(n);
            if (array[1] == 0) {
                return array;
            }
            ++i;
        }
        return array;
    }
    
    private static native byte[] WindowsRegEnumKeyEx(int p0, int p1, int p2);
    
    private static byte[] WindowsRegEnumKeyEx1(int n, int n2, int n3) {
        byte[] array = WindowsRegEnumKeyEx(n, n2, n3);
        if (array != null) {
            return array;
        }
        long millis = (long)WindowsPreferences.INIT_SLEEP_TIME;
        int i = 0;
        while (i < WindowsPreferences.MAX_ATTEMPTS) {
            try {
                Thread.sleep(millis);
            }
            catch (InterruptedException ex) {
                return array;
            }
            millis *= 2L;
            array = WindowsRegEnumKeyEx(n, n2, n3);
            if (array != null) {
                return array;
            }
            ++i;
        }
        return array;
    }
    
    private static native byte[] WindowsRegEnumValue(int p0, int p1, int p2);
    
    private static byte[] WindowsRegEnumValue1(int n, int n2, int n3) {
        byte[] array = WindowsRegEnumValue(n, n2, n3);
        if (array != null) {
            return array;
        }
        long millis = (long)WindowsPreferences.INIT_SLEEP_TIME;
        int i = 0;
        while (i < WindowsPreferences.MAX_ATTEMPTS) {
            try {
                Thread.sleep(millis);
            }
            catch (InterruptedException ex) {
                return array;
            }
            millis *= 2L;
            array = WindowsRegEnumValue(n, n2, n3);
            if (array != null) {
                return array;
            }
            ++i;
        }
        return array;
    }
    
    private WindowsPreferences(WindowsPreferences windowsPreferences, String s) {
        super(windowsPreferences, s);
        this.isBackingStoreAvailable = true;
        int openKey = windowsPreferences.openKey(4, 131097);
        if (openKey == 0) {
            this.isBackingStoreAvailable = false;
            return;
        }
        int[] windowsRegCreateKeyEx1 = WindowsRegCreateKeyEx1(openKey, toWindowsName(s));
        if (windowsRegCreateKeyEx1[1] != 0) {
            logger().warning("Could not create windows registry node " + byteArrayToString(this.windowsAbsolutePath()) + " at root 0x" + Integer.toHexString(this.rootNativeHandle()) + ". Windows RegCreateKeyEx(...) returned error code " + windowsRegCreateKeyEx1[1] + ".");
            this.isBackingStoreAvailable = false;
            return;
        }
        this.newNode = (windowsRegCreateKeyEx1[2] == 1);
        this.closeKey(openKey);
        this.closeKey(windowsRegCreateKeyEx1[0]);
        return;
    }
    
    private WindowsPreferences(int n, byte[] array) {
        super(null, "");
        this.isBackingStoreAvailable = true;
        int[] windowsRegCreateKeyEx1 = WindowsRegCreateKeyEx1(n, array);
        if (windowsRegCreateKeyEx1[1] != 0) {
            logger().warning("Could not open/create prefs root node " + byteArrayToString(this.windowsAbsolutePath()) + " at root 0x" + Integer.toHexString(this.rootNativeHandle()) + ". Windows RegCreateKeyEx(...) returned error code " + windowsRegCreateKeyEx1[1] + ".");
            this.isBackingStoreAvailable = false;
            return;
        }
        this.newNode = (windowsRegCreateKeyEx1[2] == 1);
        this.closeKey(windowsRegCreateKeyEx1[0]);
        return;
    }
    
    private byte[] windowsAbsolutePath() {
        ByteArrayOutputStream byteArrayOutputStream = new ByteArrayOutputStream();
        byteArrayOutputStream.write(WindowsPreferences.WINDOWS_ROOT_PATH, 0, WindowsPreferences.WINDOWS_ROOT_PATH.length - 1);
        StringTokenizer stringTokenizer = new StringTokenizer(this.absolutePath(), "/");
        while (stringTokenizer.hasMoreTokens()) {
            byteArrayOutputStream.write(92);
            byte[] windowsName = toWindowsName(stringTokenizer.nextToken());
            byteArrayOutputStream.write(windowsName, 0, windowsName.length - 1);
        }
        byteArrayOutputStream.write(0);
        return byteArrayOutputStream.toByteArray();
    }
    
    private int openKey(int n) {
        return this.openKey(n, n);
    }
    
    private int openKey(int n, int n2) {
        return this.openKey(this.windowsAbsolutePath(), n, n2);
    }
    
    private int openKey(byte[] array, int n, int n2) {
        if (array.length <= 257) {
            int[] array2 = WindowsRegOpenKey1(this.rootNativeHandle(), array, n);
            if (array2[1] == 5 && n2 != n) {
                array2 = WindowsRegOpenKey1(this.rootNativeHandle(), array, n2);
            }
            if (array2[1] != 0) {
                logger().warning("Could not open windows registry node " + byteArrayToString(this.windowsAbsolutePath()) + " at root 0x" + Integer.toHexString(this.rootNativeHandle()) + ". Windows RegOpenKey(...) returned error code " + array2[1] + ".");
                array2[0] = 0;
                if (array2[1] == 5) {
                    throw new SecurityException("Could not open windows registry node " + byteArrayToString(this.windowsAbsolutePath()) + " at root 0x" + Integer.toHexString(this.rootNativeHandle()) + ": Access denied");
                }
            }
            return array2[0];
        }
        else {
            return this.openKey(this.rootNativeHandle(), array, n, n2);
        }
    }
    
    private int openKey(int i, byte[] array, int n, int n2) {
        if (array.length <= 257) {
            int[] array4 = WindowsRegOpenKey1(i, array, n);
            if (array4[1] == 5 && n2 != n) {
                array4 = WindowsRegOpenKey1(i, array, n2);
            }
            if (array4[1] != 0) {
                logger().warning("Could not open windows registry node " + byteArrayToString(this.windowsAbsolutePath()) + " at root 0x" + Integer.toHexString(i) + ". Windows RegOpenKey(...) returned error code " + array4[1] + ".");
                array4[0] = 0;
            }
            return array4[0];
        }
        else {
            int n3 = -1;
            int j = 256;
            while (j > 0) {
                if (array[j] == 92) {
                    n3 = j;
                    break;
                }
                --j;
            }
            byte[] array2 = new byte[n3 + 1];
            System.arraycopy(array, 0, array2, 0, n3);
            array2[n3] = 0;
            byte[] array3 = new byte[array.length - n3 - 1];
            System.arraycopy(array, n3 + 1, array3, 0, array3.length);
            int openKey = this.openKey(i, array2, n, n2);
            if (openKey == 0) {
                return 0;
            }
            int openKey2 = this.openKey(openKey, array3, n, n2);
            this.closeKey(openKey);
            return openKey2;
        }
    }
    
    private void closeKey(int n) {
        int windowsRegCloseKey = WindowsRegCloseKey(n);
        if (windowsRegCloseKey != 0) {
            logger().warning("Could not close windows registry node " + byteArrayToString(this.windowsAbsolutePath()) + " at root 0x" + Integer.toHexString(this.rootNativeHandle()) + ". Windows RegCloseKey(...) returned error code " + windowsRegCloseKey + ".");
        }
    }
    
    @Override
    protected void putSpi(String s, String s2) {
        int openKey = this.openKey(2);
        if (openKey == 0) {
            this.isBackingStoreAvailable = false;
            return;
        }
        int windowsRegSetValueEx1 = WindowsRegSetValueEx1(openKey, toWindowsName(s), toWindowsValueString(s2));
        if (windowsRegSetValueEx1 != 0) {
            logger().warning("Could not assign value to key " + byteArrayToString(toWindowsName(s)) + " at Windows registry node " + byteArrayToString(this.windowsAbsolutePath()) + " at root 0x" + Integer.toHexString(this.rootNativeHandle()) + ". Windows RegSetValueEx(...) returned error code " + windowsRegSetValueEx1 + ".");
            this.isBackingStoreAvailable = false;
        }
        this.closeKey(openKey);
        return;
    }
    
    @Override
    protected String getSpi(String s) {
        int openKey = this.openKey(1);
        if (openKey == 0) {
            return null;
        }
        byte[] windowsRegQueryValueEx = WindowsRegQueryValueEx(openKey, toWindowsName(s));
        if (windowsRegQueryValueEx == null) {
            this.closeKey(openKey);
            return null;
        }
        this.closeKey(openKey);
        return toJavaValueString((byte[])(byte[])windowsRegQueryValueEx);
    }
    
    @Override
    protected void removeSpi(String s) {
        int openKey = this.openKey(2);
        if (openKey == 0) {
            return;
        }
        int windowsRegDeleteValue = WindowsRegDeleteValue(openKey, toWindowsName(s));
        if (windowsRegDeleteValue != 0 && windowsRegDeleteValue != 2) {
            logger().warning("Could not delete windows registry value " + byteArrayToString(this.windowsAbsolutePath()) + "\\" + toWindowsName(s) + " at root 0x" + Integer.toHexString(this.rootNativeHandle()) + ". Windows RegDeleteValue(...) returned error code " + windowsRegDeleteValue + ".");
            this.isBackingStoreAvailable = false;
        }
        this.closeKey(openKey);
        return;
    }
    
    @Override
    protected String[] keysSpi() throws BackingStoreException {
        int openKey = this.openKey(1);
        if (openKey == 0) {
            throw new BackingStoreException("Could not open windows registry node " + byteArrayToString(this.windowsAbsolutePath()) + " at root 0x" + Integer.toHexString(this.rootNativeHandle()) + ".");
        }
        int[] windowsRegQueryInfoKey1 = WindowsRegQueryInfoKey1(openKey);
        if (windowsRegQueryInfoKey1[1] != 0) {
            String string2 = "Could not query windows registry node " + byteArrayToString(this.windowsAbsolutePath()) + " at root 0x" + Integer.toHexString(this.rootNativeHandle()) + ". Windows RegQueryInfoKeyEx(...) returned error code " + windowsRegQueryInfoKey1[1] + ".";
            logger().warning(string2);
            throw new BackingStoreException(string2);
        }
        int n = windowsRegQueryInfoKey1[4];
        int n2 = windowsRegQueryInfoKey1[2];
        if (n2 == 0) {
            this.closeKey(openKey);
            return new String[0];
        }
        String[] array = new String[n2];
        int i = 0;
        while (i < n2) {
            byte[] windowsRegEnumValue1 = WindowsRegEnumValue1(openKey, i, n + 1);
            if (windowsRegEnumValue1 == null) {
                String string = "Could not enumerate value #" + i + "  of windows node " + byteArrayToString(this.windowsAbsolutePath()) + " at root 0x" + Integer.toHexString(this.rootNativeHandle()) + ".";
                logger().warning(string);
                throw new BackingStoreException(string);
            }
            array[i] = toJavaName(windowsRegEnumValue1);
            ++i;
        }
        this.closeKey(openKey);
        return array;
    }
    
    @Override
    protected String[] childrenNamesSpi() throws BackingStoreException {
        int openKey = this.openKey(9);
        if (openKey == 0) {
            throw new BackingStoreException("Could not open windows registry node " + byteArrayToString(this.windowsAbsolutePath()) + " at root 0x" + Integer.toHexString(this.rootNativeHandle()) + ".");
        }
        int[] windowsRegQueryInfoKey1 = WindowsRegQueryInfoKey1(openKey);
        if (windowsRegQueryInfoKey1[1] != 0) {
            String string2 = "Could not query windows registry node " + byteArrayToString(this.windowsAbsolutePath()) + " at root 0x" + Integer.toHexString(this.rootNativeHandle()) + ". Windows RegQueryInfoKeyEx(...) returned error code " + windowsRegQueryInfoKey1[1] + ".";
            logger().warning(string2);
            throw new BackingStoreException(string2);
        }
        int n = windowsRegQueryInfoKey1[3];
        int n2 = windowsRegQueryInfoKey1[0];
        if (n2 == 0) {
            this.closeKey(openKey);
            return new String[0];
        }
        String[] array = new String[n2];
        String[] array2 = new String[n2];
        int i = 0;
        while (i < n2) {
            byte[] windowsRegEnumKeyEx1 = WindowsRegEnumKeyEx1(openKey, i, n + 1);
            if (windowsRegEnumKeyEx1 == null) {
                String string = "Could not enumerate key #" + i + "  of windows node " + byteArrayToString(this.windowsAbsolutePath()) + " at root 0x" + Integer.toHexString(this.rootNativeHandle()) + ". ";
                logger().warning(string);
                throw new BackingStoreException(string);
            }
            array2[i] = toJavaName(windowsRegEnumKeyEx1);
            ++i;
        }
        this.closeKey(openKey);
        return array2;
    }
    
    @Override
    public void flush() throws BackingStoreException {
        if (this.isRemoved()) {
            this.parent.flush();
            return;
        }
        if (!this.isBackingStoreAvailable) {
            throw new BackingStoreException("flush(): Backing store not available.");
        }
        int openKey = this.openKey(131097);
        if (openKey == 0) {
            throw new BackingStoreException("Could not open windows registry node " + byteArrayToString(this.windowsAbsolutePath()) + " at root 0x" + Integer.toHexString(this.rootNativeHandle()) + ".");
        }
        int windowsRegFlushKey1 = WindowsRegFlushKey1(openKey);
        if (windowsRegFlushKey1 != 0) {
            String string = "Could not flush windows registry node " + byteArrayToString(this.windowsAbsolutePath()) + " at root 0x" + Integer.toHexString(this.rootNativeHandle()) + ". Windows RegFlushKey(...) returned error code " + windowsRegFlushKey1 + ".";
            logger().warning(string);
            throw new BackingStoreException(string);
        }
        this.closeKey(openKey);
        return;
    }
    
    @Override
    public void sync() throws BackingStoreException {
        if (this.isRemoved()) {
            throw new IllegalStateException("Node has been removed");
        }
        this.flush();
        return;
    }
    
    @Override
    protected AbstractPreferences childSpi(String s) {
        return new WindowsPreferences(this, s);
    }
    
    public void removeNodeSpi() throws BackingStoreException {
        int openKey = ((WindowsPreferences)this.parent()).openKey(65536);
        if (openKey == 0) {
            throw new BackingStoreException("Could not open parent windows registry node of " + byteArrayToString(this.windowsAbsolutePath()) + " at root 0x" + Integer.toHexString(this.rootNativeHandle()) + ".");
        }
        int windowsRegDeleteKey = WindowsRegDeleteKey(openKey, toWindowsName(this.name()));
        if (windowsRegDeleteKey != 0) {
            String string = "Could not delete windows registry node " + byteArrayToString(this.windowsAbsolutePath()) + " at root 0x" + Integer.toHexString(this.rootNativeHandle()) + ". Windows RegDeleteKeyEx(...) returned error code " + windowsRegDeleteKey + ".";
            logger().warning(string);
            throw new BackingStoreException(string);
        }
        this.closeKey(openKey);
        return;
    }
    
    private static String toJavaName(byte[] array) {
        String byteArrayToString = byteArrayToString(array);
        if (byteArrayToString.length() > 1 && byteArrayToString.substring(0, 2).equals("/!")) {
            return toJavaAlt64Name(byteArrayToString);
        }
        StringBuilder sb = new StringBuilder();
        for (int i = 0; i < byteArrayToString.length(); ++i) {
            int char1;
            if ((char1 = byteArrayToString.charAt(i)) == '/') {
                int char2 = ' ';
                if (byteArrayToString.length() > i + 1 && (char2 = byteArrayToString.charAt(i + 1)) >= 'A' && char2 <= 'Z') {
                    char1 = char2;
                    ++i;
                }
                else if (byteArrayToString.length() > i + 1 && char2 == '/') {
                    char1 = '\\';
                    ++i;
                }
            }
            else if (char1 == '\\') {
                char1 = '/';
            }
            sb.append((char)char1);
        }
        return sb.toString();
    }
    
    private static String toJavaAlt64Name(String s) {
        byte[] altBase64ToByteArray = Base64.altBase64ToByteArray(s.substring(2));
        StringBuilder sb = new StringBuilder();
        for (int i = 0; i < altBase64ToByteArray.length; ++i) {
            sb.append((char)(((altBase64ToByteArray[i++] & 0xFF) << 8) + (altBase64ToByteArray[i] & 0xFF)));
        }
        return sb.toString();
    }
    
    private static byte[] toWindowsName(String s) {
        StringBuilder sb = new StringBuilder();
        int i = 0;
        while (i < s.length()) {
            int char1 = s.charAt(i);
            if (char1 < ' ' || char1 > '\u007f') {
                return toWindowsAlt64Name(s);
            }
            if (char1 == '\\') {
                sb.append("//");
            }
            else if (char1 == '/') {
                sb.append('\\');
            }
            else if (char1 >= 'A' && char1 <= 'Z') {
                sb.append('/').append((char)char1);
            }
            else {
                sb.append((char)char1);
            }
            ++i;
        }
        return stringToByteArray(sb.toString());
    }
    
    private static byte[] toWindowsAlt64Name(String s) {
        byte[] array = new byte[2 * s.length()];
        int n = 0;
        for (int i = 0; i < s.length(); ++i) {
            int char1 = s.charAt(i);
            array[n++] = (byte)(char1 >>> 8);
            array[n++] = (byte)char1;
        }
        return stringToByteArray("/!" + Base64.byteArrayToAltBase64(array));
    }
    
    private static String toJavaValueString(byte[] array) {
        String byteArrayToString = byteArrayToString(array);
        StringBuilder sb = new StringBuilder();
        for (int i = 0; i < byteArrayToString.length(); ++i) {
            int char1;
            if ((char1 = byteArrayToString.charAt(i)) == '/') {
                int char2 = ' ';
                if (byteArrayToString.length() > i + 1 && (char2 = byteArrayToString.charAt(i + 1)) == 'u') {
                    if (byteArrayToString.length() < i + 6) {
                        break;
                    }
                    char1 = (char)Integer.parseInt(byteArrayToString.substring(i + 2, i + 6), 16);
                    i += 5;
                }
                else if (byteArrayToString.length() > i + 1 && byteArrayToString.charAt(i + 1) >= 'A' && char2 <= 'Z') {
                    char1 = char2;
                    ++i;
                }
                else if (byteArrayToString.length() > i + 1 && char2 == '/') {
                    char1 = '\\';
                    ++i;
                }
            }
            else if (char1 == '\\') {
                char1 = '/';
            }
            sb.append((char)char1);
        }
        return sb.toString();
    }
    
    private static byte[] toWindowsValueString(String s) {
        StringBuilder sb = new StringBuilder();
        for (int i = 0; i < s.length(); ++i) {
            int char1 = s.charAt(i);
            if (char1 < ' ' || char1 > '\u007f') {
                sb.append("/u");
                StringBuilder o = new StringBuilder(Integer.toHexString((int)s.charAt(i)));
                o.reverse();
                for (int n = 4 - o.length(), j = 0; j < n; ++j) {
                    o.append('0');
                }
                for (int k = 0; k < 4; ++k) {
                    sb.append(o.charAt(3 - k));
                }
            }
            else if (char1 == '\\') {
                sb.append("//");
            }
            else if (char1 == '/') {
                sb.append('\\');
            }
            else if (char1 >= 'A' && char1 <= 'Z') {
                sb.append('/').append((char)char1);
            }
            else {
                sb.append((char)char1);
            }
        }
        return stringToByteArray(sb.toString());
    }
    
    private int rootNativeHandle() {
        return this.isUserNode() ? -2147483647 : -2147483646;
    }
    
    private static byte[] stringToByteArray(String s) {
        byte[] array = new byte[s.length() + 1];
        for (int i = 0; i < s.length(); ++i) {
            array[i] = (byte)s.charAt(i);
        }
        array[s.length()] = 0;
        return array;
    }
    
    private static String byteArrayToString(byte[] array) {
        StringBuilder sb = new StringBuilder();
        for (int i = 0; i < array.length - 1; ++i) {
            sb.append((char)array[i]);
        }
        return sb.toString();
    }
    
    @Override
    protected void flushSpi() throws BackingStoreException {
    }
    
    @Override
    protected void syncSpi() throws BackingStoreException {
    }
    
    private static synchronized PlatformLogger logger() {
        if (WindowsPreferences.logger == null) {
            WindowsPreferences.logger = PlatformLogger.getLogger("java.util.prefs");
        }
        return WindowsPreferences.logger;
    }
    
    static {
        WINDOWS_ROOT_PATH = stringToByteArray("Software\\JavaSoft\\Prefs");
        WindowsPreferences.INIT_SLEEP_TIME = 50;
        WindowsPreferences.MAX_ATTEMPTS = 5;
    }
}
